#include "stdafx.h"
#include "LottoMan.h"
#include "LottoManDoc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif



CMember::CMember()
{
	m_bMarkedForDeletion = FALSE;

  m_Numbers.SetSize(6);
  m_Numbers.SetAt(0, 0);
  m_Numbers.SetAt(1, 0);
  m_Numbers.SetAt(2, 0);
  m_Numbers.SetAt(3, 0);
  m_Numbers.SetAt(4, 0);
  m_Numbers.SetAt(5, 0);

  m_bEnabled = TRUE;
}

CMember::CMember(const CMember& member)
{
	*this = member;
}

CMember::~CMember()
{
}

CMember& CMember::operator=(const CMember& member)
{
	m_sName = member.m_sName;
  m_Numbers.Copy(member.m_Numbers);
  m_bEnabled = member.m_bEnabled;
	m_bMarkedForDeletion = member.m_bMarkedForDeletion;

	return *this;
}

void CMember::Serialize(CArchive& ar)
{
  //let the parent do its stuff
  CObject::Serialize(ar);

  ASSERT(!m_bMarkedForDeletion);

  //Serialize the elements
  if (ar.IsLoading())
  {
    WORD wConfigVersion;
    ar >> wConfigVersion;
    ar >> m_sName;

    for (int i=0; i<m_Numbers.GetSize(); i++)
    {
      int nTemp;
      ar >> nTemp;
      m_Numbers.SetAt(i, nTemp);
    }

    if (wConfigVersion > 0x100)
      ar >> m_bEnabled;
    else
      m_bEnabled = TRUE;
  }
  else
  {
    WORD wConfigVersion = 0x101;
    ar << wConfigVersion;
    ar << m_sName;


		for (int i=0; i<m_Numbers.GetSize(); i++)
			ar << m_Numbers.GetAt(i);

    ar << m_bEnabled;
  }
}





IMPLEMENT_DYNCREATE(CLottoManDoc, CDocument)

BEGIN_MESSAGE_MAP(CLottoManDoc, CDocument)
	//{{AFX_MSG_MAP(CLottoManDoc)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

CLottoManDoc::CLottoManDoc()
{
  m_nListViewStyle = 3;
  m_nMaxLottoNumber = 42;
  m_nNumbersInDraw = 6;
  m_bBonusNumberPicked = TRUE;
}

CLottoManDoc::~CLottoManDoc()
{
  FreeAllMemory();
}

void CLottoManDoc::Serialize(CArchive& ar)
{
  if (ar.IsLoading())
  {
    FreeAllMemory();

    //Version Stamp
    WORD wConfigVersion;
    ar >> wConfigVersion;

    //the members
    int nMembers;
    ar >> nMembers;
    m_Members.SetSize(nMembers);
    for (int i=0; i<nMembers; i++)
    {
      CMember* pMember = new CMember;
      pMember->m_Numbers.SetSize(6); //Hard coded to 6 numbers to preserve the current file format
      pMember->Serialize(ar);
      m_Members.SetAt(i, pMember);
    }

    //The list view style
    if (wConfigVersion > 0x100)
      ar >> m_nListViewStyle;

    if (wConfigVersion > 0x101)
    {
      ar >> m_nMaxLottoNumber;
      ar >> m_nNumbersInDraw;
      ar >> m_bBonusNumberPicked;
    }
  }
  else
  {
    //Version Stamp
    WORD wConfigVersion = 0x102;
    ar << wConfigVersion;

    //The date ranges
    ar << NumberOfMembers();
    for (int i=0; i<m_Members.GetSize(); i++)
    {
      CMember* pMember = m_Members.GetAt(i);
      if (!pMember->m_bMarkedForDeletion)
        pMember->Serialize(ar);
    }

    
    ar << m_nListViewStyle; //The list view style

    ar << m_nMaxLottoNumber; //The new fields
    ar << m_nNumbersInDraw;
    ar << m_bBonusNumberPicked;
  }
}

#ifdef _DEBUG
void CLottoManDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CLottoManDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

void CLottoManDoc::FreeAllMemory()
{
  for (int i=0; i<m_Members.GetSize(); i++)
    delete m_Members.GetAt(i);
  m_Members.RemoveAll();
}

void CLottoManDoc::MarkMemberAsDeleted(int nIndex)
{
  CMember* pMember = m_Members.GetAt(nIndex);
  pMember->m_bMarkedForDeletion = TRUE;
}

int CLottoManDoc::NumberOfMembers() const
{
  int nMembers = 0;
  for (int i=0; i<m_Members.GetSize(); i++)
  {
    CMember* pMember = m_Members.GetAt(i);
    if (!pMember->m_bMarkedForDeletion)
      nMembers++;
  }
  return nMembers;
}

CString CLottoManDoc::CreateUniqueMemberName() const
{
  CString sName;
  BOOL bExist = TRUE;
  int nNumber = (m_Members.GetSize() + 1);
  while (bExist)
  {
    CString sNum;
    sNum.Format(_T("%d"), nNumber);
    AfxFormatString1(sName, IDS_NEW_MEMBER, sNum);
    bExist = DoesMemberNameExist(sName, -1);
    nNumber++;
  }

  return sName;
}

BOOL CLottoManDoc::DoesMemberNameExist(const CString& sName, int nExistingIndex) const
{
  for (int i=0; i<m_Members.GetSize(); i++)
  {
    CMember* pMember = m_Members.GetAt(i);
    if (!pMember->m_bMarkedForDeletion && (i != nExistingIndex) && (pMember->m_sName.CompareNoCase(sName) == 0))
      return TRUE;
  }
  return FALSE;
}

CMember* CLottoManDoc::GetAtMember(int nMemberNum) const
{
  int nValidMembers = -1;
  int nOffset = 0;
  while (nValidMembers < nMemberNum)
  {
    if (m_Members.GetAt(nOffset)->m_bMarkedForDeletion)
      nOffset++;
    else
    {
      nValidMembers++;
      nOffset++;
    }
  }

  return m_Members.GetAt(nOffset-1);
}



void CLottoManDoc::DeleteContents() 
{
	FreeAllMemory();
	
	CDocument::DeleteContents();
}
